<?php

# Quality security! <3

/**
* @brief Get the visitor IP address
* @return IP, 0.0.0.0 if not found
 */
function getIp()
{
	static $vars = array(
		'HTTP_FORWARDED_FOR', 'HTTP_X_FORWARDED_FOR','HTTP_X_COMING_FROM',
		'HTTP_VIA', 'HTTP_XROXY_CONNECTION', 'HTTP_CLIENT_IP', 'REMOTE_ADDR'
	);

	foreach ($vars as $var)
	{
		if (isset($_SERVER[$var])) {
			$val = $_SERVER[$var];
			$val = explode(',', $val);
			$val = str_replace(' ', '', $val[0]);

			if (filter_var($val, FILTER_VALIDATE_IP)) {
				return $val;
			}
		}
	}

	return '0.0.0.0';
}

/**
* @brief Get the value at $array key $varname, if it exists and is a string.
* @return String if (string) $varname in $array, else NULL
*/
function getString($array, $varname)
{
	if (isset($array[$varname]) && is_string($array[$varname])) {
		return $array[$varname];
	}

	return NULL;
}

/**
* @brief Safely return a GET value
* @return String if GET key $varname exists, else NULL
*/
function getGet($varname)
{
	return getString($_GET, $varname);
}

/**
* @brief Safely return a POST value
* @return String if POST key $varname exists, else NULL
*/
function getPost($varname)
{
	return getString($_POST, $varname);
}

/**
* @brief return a SESSION value
* @return String if SESSION key $varname exists, else NULL
*/
function getSession($varname)
{
	if (isset($_SESSION[$varname]))
		return $_SESSION[$varname];

	return NULL;
}

/**
* @brief Simply set a session variable
*/
function setSession($varname, $value)
{
	$_SESSION[$varname] = $value;
}

/**
* @brief Convert characters to entities to avoid XSS problems
* @return $var converted with entities
*/
function safeHTML($var)
{
	return htmlentities($var);
}

/**
* @brief Generate an anti-CSRF token. Store it in $_SESSION.
* @return a random token string
*/
function getToken($var)
{
	$charset = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789[]_-()={}+*@^:;,!.";
	$mod = strlen($charset);

	$result = '';

	for ($i = 0; $i < 10; ++$i) {
		$result .= $charset[rand(0, 100000) % $mod];
	}

	$_SESSION[$var] = $result;

	return $result;
}

/**
* @brief Check there is a recorded token for $name, and that it's value is
*        the same found in $tab (wich defaults to $_POST).
*        Unset the token in $_SESSION.
* @param $name Name of the token
* @param $tab Array to compare with $_SESSION. Default is $_POST
* @return Boolean TRUE if token is OK
*/
function checkToken($name, $tab=NULL)
{
	if (is_null($tab)) {
		$tab = $_POST;
	}

	if (isset($_SESSION[$name], $tab, $tab[$name]) && $_SESSION[$name] == $tab[$name]) {
		unset($_SESSION[$name]);
		return TRUE;
	}

	return FALSE;
}

/**
* @brief Remove all spaces and the like characters (newline, carriage return, null, tab)
*
* return Cleaned string
*/
function removeAllSpaces($txt)
{
	return preg_replace('/[\t\n\0\r ]/', '', $txt);
}

/**
* @brief Salt the password with a fix salt + login
*        Use sha256 hash.
* @return Salted password hash
*/
function getPassword($login, $password)
{
	static $salt = 'z8Y ;Zorece%`*!E$o}6u.8,*yN}MdqZ9.RK*hEy!&';

	return hash('sha512', $salt . $password . $login);
}

/**
* @brief Try to detect XSS attempts. If a XSS keyword is detected, try filter2 regex.
* @return True if XSS detected, else False
*/
function detectXSS($input)
{
	$filter1 = '(jscript|onsubmit|copyparentfolder|javascript|meta|onmove|'.
	           'onkeydown|onchange|onkeyup|activexobject|expression|onmouseup|'.
	           'ecmascript|onmouseover|vbscript:|<!\[cdata\[|http:|settimeout|onabort|'.
	           'shell:|\.innerhtml|onmousedown|onkeypress|asfunction:|onclick|'.
	           '\.fromcharcode|background-image:|\.cookie|ondragdrop|onblurx-javascript|'.
	           'mocha:|onfocus|javascript:|getparentfolder|lowsrc|onresize|\@import|'.
	           'alert|onselect|script|onmouseout|onmousemove|background|application|'.
	           '\.execscript|livescript:|getspecialfolder|vbscript|iframe|\.addimport|'.
	           'onunload|createtextrange|onload|<input)';

	$filter2 = '(?:\b(?:(?:type\b\W*?\b(?:text\b\W*?\b'.
	           '(?:j(?:ava)?|ecma|vb)|application\b\W*?\bx-(?:java|vb))script|c(?:opyparentfolde'.
	           'r|reatetextrange)|get(?:special|parent)folder|iframe\b.{0,100}?\bsrc)\b|on(?:(?:'.
	           'mo(?:use(?:o(?:ver|ut)|down|move|up)|ve)|key(?:press|down|up)|c(?:hange|lick)|s('.
	           '?:elec|ubmi)t|(?:un)?load|dragdrop|resize|focus|blur)\b\W*?=|abort\b)|(?:l(?:ows'.
	           'rc\b\W*?\b(?:(?:java|vb)script|shell|http)|ivescript)|(?:href|url)\b\W*?\b(?:(?:'.
	           'java|vb)script|shell)|background-image|mocha):|s(?:(?:tyle\b\W*=.*\bexpression\b'.
	           '\W*|ettimeout\b\W*?)\(|rc\b\W*?\b(?:(?:java|vb)script|shell|http):)|a(?:ctivexob'.
	           'ject\b|lert\b\W*?\(|sfunction:))|<(?:(?:body\b.*?\b(?:backgroun|onloa)d|input\b.'.
	           '*?\btype\b\W*?\bimage)\b| ?(?:(?:script|meta)\b|iframe)|!\[cdata\[)|(?:\.(?:(?:e'.
	           'xecscrip|addimpor)t|(?:fromcharcod|cooki)e|innerhtml)|\@import)\b)';

	return (preg_match('/'.$filter1.'/i',$input) && preg_match('/'.$filter2.'/i', $input));
}

/**
* @brief Try to detect SQL injection attempts.
* @return True if SQL injection detected, else False
*/
function detectSQLi($input)
{
	$filter = '(?:^\s*[;>"]\s*(?:union|select|create|rename|truncate|load|alter|delete|'.
	          'updateiinsert|desc))|(?:(?:select|create|rename|truncate|load|alter|delete|'.
	          'update|insert|desc)\s+(?:concat|char|load_file)\s?\(?)|(?:end\s*\);)|("\s+regexp\W)';

	return preg_match('/'.$filter.'/i',$input);
}

/**
* @brief Check that the request URI doesn't begin with '?', 'index.php', 'index'.
*        The goal is to force utilisation of the URL Rewritting system.
* @return Nothing. Violent death for hackerz.
*/
function checkRequestURI()
{
	if (preg_match('/^\/(?:\?|index)/i', $_SERVER['REQUEST_URI'])) {
		header('Location: /');
		die('Johny, la gente esta muy loca. WTF.');
	}
}

?>
