package johndog.core;

import java.security.*;
import java.security.cert.*;
import java.io.*;
import java.net.*;
import java.util.jar.*;

public class JDClassLoader extends URLClassLoader {
	private String siteBase;
	
	public JDClassLoader(URLClassLoader delegate) {
		super(new URL[0], null);
		this.siteBase = null;
		
		URL[] baseClassPath = ((URLClassLoader)delegate).getURLs();
		for (int i=0;i < baseClassPath.length;i++)
			this.addURL(baseClassPath[i]);
	}
	
	public void setSiteBase(String siteBase) {
		this.siteBase = siteBase;
	}
	
	protected Class findClass(String name) throws ClassNotFoundException {
		/* Bootstrap class emulation */
		if (name.equals(this.getClass().getName())) {
			return this.getClass();
		}
		
		/* 
		 * Find classes in inherited class path
		 * If none found, load a sandboxed site class
		 */
		try {
			return super.findClass(name);
		} catch (ClassNotFoundException cnfe) {
			try {
				return this.loadSiteClass(name);
			} catch (Throwable th) {
				throw new ClassNotFoundException();
			}
		}
	}

	private Class loadSiteClass(String name) throws Exception, Error {
		if (this.siteBase == null)
			throw new ClassNotFoundException();
		 
		String filePath = name;
		String fileSep = (String)AccessController.doPrivileged(
							new PrivilegedAction() {
								public String run() {
									return System.getProperty("file.separator");
								}
							});
		filePath.replace(".", fileSep);

		JarFile jar = new JarFile(siteBase);
		JarEntry je = jar.getJarEntry(filePath + ".class");
		InputStream jis = jar.getInputStream(je);
		byte[] classArray = new byte[(int)je.getSize()];
		
		jis.read(classArray);

		/* Don't trash other websites */
		Permissions permissions = new Permissions();
		permissions.add(new FilePermission(siteBase, "read"));
			
		return defineClass(name, classArray, 0, classArray.length, new ProtectionDomain(new CodeSource(new URL("file:" + siteBase), new java.security.cert.Certificate[0]), permissions));
	}
	
}
