package johndog.core;

import java.net.*;
import java.io.*;
import java.util.*;

class JDHTTPRequest {

	private Map<String, String> headers;
	private Map<String, String> params;
	private String method;
	private String protocol;
	private String url;
	private String path;
	
	public JDHTTPRequest() {
		headers = new HashMap<String, String>();
		params = new HashMap<String, String>();
		method = protocol = url = null;
	}
	
	public void parseRequest(BufferedReader input) throws IOException {
		String line = input.readLine();
		if (line == null)
			return;

		StringTokenizer request = new StringTokenizer(line);
		if (request.hasMoreTokens())
			method = request.nextToken();
		if (request.hasMoreTokens())
			url = request.nextToken();
		if (request.hasMoreTokens())
			protocol = request.nextToken();
	}
	
	public void parseHeaders(BufferedReader input) throws IOException {			
		String line;
		int idx;
		
		while ((line = input.readLine()) != null) {
			idx = line.indexOf(':');
			if (idx == -1) // Assume \r\n
				break;
			if (idx == 0)
				continue;
			String headerValue = line.substring(idx+1, line.length()).trim();
			if (headerValue.length() > 0)
				headers.put(line.substring(0, idx), headerValue);
		}
	}
	
	public void parseData(BufferedReader input) throws IOException {
		String line;
		int idx;
		
		if ((line = headers.get("Content-Length")) != null) {
			char cbuf[] = new char[Integer.parseInt(line)];
			input.read(cbuf, 0, Integer.parseInt(line));
			params.put("POST", new String(cbuf));
		} else {
			params.put("POST", null);
		}
		
		line = null;
		idx = url.indexOf("?");
		if (idx != -1) {				
			line = url.substring(idx+1, url.length());
			path = url.substring(0, idx);
		} else {
			path = url;
		}
		params.put("GET", line);
	}
	
	public void doResponse(DataOutputStream output) {
		JDHTTPResponse response = new JDHTTPResponse();
		
		response.validateRequest(method, path, protocol);
		response.validatePath(path, headers);
		response.getResource(headers, params);
		
		response.doResponse(output);
	}

}
