package johndog.util;

import java.util.*;
import java.lang.reflect.*;

public class JDHTTPContext {
	
	private Map<String, String> params;
	private Map<String, String> headers;
	private Map<String, Object> sessions;
	private Object site;
	private String method;

	public JDHTTPContext() {
		this.params = new HashMap<String, String>();
		this.headers = new HashMap<String, String>();
		this.sessions = new HashMap<String, Object>();
		this.site = null;
		this.method = null;
	}

	public void addHeaders(Map<String, String> headers) {
		this.headers.putAll(headers);		
	}
	public void addSessions(Map<String, Object> sessions) {
		this.sessions.putAll(sessions);		
	}
	public void addParams(String params) {
		if (params != null)
			for (String param : params.split("&")) {
				String pair[] = param.split("=");
				String key = pair[0];
				String value = "";
				if (pair.length > 1) {
					value = pair[1];
				}
				this.params.put(key, value);
			}
	}
	
	public String getHeader(String name) {
		return this.headers.get(name);
	}
	public String getParam(String name) {
		return this.params.get(name);
	}
	public Object getSession(String sessid) {
		return this.sessions.get(sessid);
	}
	
	public void loadWebSite(Class siteClass, String method) throws Exception {
		this.method = method;
		
		try {
			this.site = siteClass.getDeclaredConstructor(new Class[]{JDHTTPContext.class}).newInstance(this);
		} catch (Exception ex) {
			try {
				this.site = siteClass.newInstance();
			} catch(Exception ex2) {}
		}
	}
	
	public Object getValue() throws Exception {
		Object ret = null;
		
		if (site != null) {
			Method m = (site instanceof Class ? (Class)site : site.getClass()).getMethod(method);			
			ret = m.invoke(site, new Object[0]);
		}
		
		return ret;
	}

}
